/*
 * HtmlPainter.cpp
 *
 *  Created on: 30.7.2010
 *      Author: sber
 */

#include "HtmlPainter.h"

HtmlPainter::HtmlPainter(Type type, bool oneLineMode)
  : oneLineMode(oneLineMode)
  , ellipsisWidth(0)
  , showEllipsis(false)
  , previousWidth(-1)
  , previousSize()
  , label(new QLabel())
  , ellipsisLabel(new QLabel())
{
    switch(type) {
        case Title:
            label->setObjectName("HtmlPainterTitle");
            ellipsisLabel->setObjectName("HtmlPainterTitle");
            break;
        case Content:
            label->setObjectName("HtmlPainterContent");
            ellipsisLabel->setObjectName("HtmlPainterContent");
            break;
        case Info:
            label->setObjectName("HtmlPainterInfo");
            ellipsisLabel->setObjectName("HtmlPainterInfo");
            break;
        case Detail:
            label->setObjectName("HtmlPainterDetail");
            ellipsisLabel->setObjectName("HtmlPainterDetail");
            break;
    }
    
    label->setTextFormat(Qt::RichText);
    
    ellipsisLabel->setText(QObject::tr("..."));
    ellipsisWidth = ellipsisLabel->sizeHint().width();
    
    QString const style("background-color: transparent;");
    label->setStyleSheet(style);
    ellipsisLabel->setStyleSheet(style);
}

HtmlPainter::~HtmlPainter()
{
    
}


void HtmlPainter::setContent(QString const & content)
{
    label->setText(content);
}

QSize HtmlPainter::setWidthAndMode(int width, bool oneLine)
{
    // Return if there is no change
    if(oneLineMode == oneLine && previousWidth == width) {
        return previousSize;
    }
    
    oneLineMode = oneLine;
    previousWidth = width;
    
    int finalHeight = 0;
    
    if(oneLineMode) {
        label->setWordWrap(false);
        finalHeight = label->sizeHint().height();
        
        int textWidth = label->sizeHint().width();
        if(textWidth > width) {
            // Elide
            showEllipsis = true;
            int const labelWidth = width - ellipsisWidth - 2; // make a few pixel gap
            label->setGeometry(QRect(QPoint(), QSize(labelWidth, finalHeight)));
            ellipsisLabel->setGeometry(QRect(QPoint(labelWidth, 0), QSize(ellipsisWidth, finalHeight)));
        } else {
            // No need to elide
            showEllipsis = false;
            label->setGeometry(QRect(QPoint(), QSize(width, finalHeight)));
        }
    } else {
        // Show whole text
        label->setWordWrap(true);
        finalHeight = label->heightForWidth(width);
        label->setGeometry(QRect(QPoint(), QSize(width, finalHeight)));
    }
    
    previousSize = QSize(width, finalHeight); 
    return previousSize;
}

void HtmlPainter::paint(QPainter * painter, QRectF const & rect)
{
    renderInRect(label.data(), painter, rect);
    if(oneLineMode && showEllipsis) {
        renderInRect(ellipsisLabel.data(), painter, rect);
    }
}

void HtmlPainter::renderInRect(QWidget * widget, QPainter * painter, QRectF const & rect)
{
    QPoint offset(rect.topLeft().x(), rect.topLeft().y());
    QPoint point = widget->geometry().topLeft() + offset;
    widget->render(painter, point, QRegion(QRect(QPoint(), widget->size())));
}
