#include "LocationButton.h"

#include <QFontMetrics>
#include <QLayout>

using QtMobility::QGeoCoordinate;

LocationButton::LocationButton(QWidget * parent)
  : QPushButton(parent)
    {
    resetCoordinate();
    }

void LocationButton::resetCoordinate()
    {
    coordinate = QGeoCoordinate();
    updateText();
    }

void LocationButton::setCoordinate(QtMobility::QGeoCoordinate const & newCoordinate)
    {
    coordinate = newCoordinate;
    updateText();
    }

void LocationButton::resizeEvent(QResizeEvent* event)
    {
    QPushButton::resizeEvent(event);
    updateText();
    }

void LocationButton::updateText()
    {
    if(!coordinate.isValid()) {
        setText(tr("Add location..."));
        return;
    }
    
    int maxWidth = getTextWidth();
    QFontMetrics fm = fontMetrics();
    QString coordStr = coordinate.toString(QGeoCoordinate::DegreesMinutesSecondsWithHemisphere);
    if(fm.width(coordStr) > maxWidth) {
        coordStr = coordinate.toString(QGeoCoordinate::DegreesMinutesWithHemisphere);
        if(fm.width(coordStr) > maxWidth) {
            coordStr = coordinate.toString(QGeoCoordinate::DegreesWithHemisphere);
        }
    }
    
    /*: %1 is a location coordinate, which has to be on it's own line
     * e.g. 57 18' 22.5" N, 4 27' 32.7" W
     * This REALLY needs to be on it's own line, or the whole view might break!
     */
    setText(tr("Remove location:\n%1").arg(coordStr));
    }

int LocationButton::getTextWidth()
    {
    /* XXX how the heck do we really find out the available text width?
     * Worst case scenario is that the text overflows making the 
     */
    
    // assume max one em of padding
    QFontMetrics fm = fontMetrics();
    int em = fm.height();
    return contentsRect().width() - 2 * em;
    }
