#include "wpbloggraphicslistitem.h"
#include "WPData.h"

WPBlogGraphicsListItem::Geometry WPBlogGraphicsListItem::iGeometry;

WPBlogGraphicsListItem::WPBlogGraphicsListItem(const WPDataBlog& blog, BusyIndicator& busyIndicator, ThemeData& themeData)
  : WPGraphicsListItem(busyIndicator, themeData)
  , iBlog(blog)
  , iTitle(blog.iBlogName)
{
	int draftPosts = 0;
	int unapprovedComments = 0;
	QDateTime latestActivity;
	QDateTime comparisonDate;

	for (QListIterator<WPPost> i = blog.GetPosts(); i.hasNext(); ) {
		const WPPost post = i.next();

		if (post->GetLocalTime() > latestActivity)
			latestActivity = post->GetLocalTime();

		if (post->IsDraft())
			draftPosts++;
	}

	for (QListIterator<WPComment> j = blog.GetComments(); j.hasNext(); ) {
		WPComment comment = j.next();

		if (comment->GetLocalTime() > latestActivity)
			latestActivity = comment->GetLocalTime();

		if (comment->IsUnapproved())
			unapprovedComments++;
	}

	if (unapprovedComments == 1)
		iStatus += tr("1 unapproved comment");
	else if (unapprovedComments > 1)
		iStatus += QString::number(unapprovedComments) + tr(" unapproved comments");

	if (draftPosts > 0)
	{
		if (!iStatus.isEmpty())
			iStatus += ", ";
		if (draftPosts == 1)
			iStatus += tr("1 draft");
		else
			iStatus += QString::number(draftPosts) + tr(" drafts");
	}

	iActivity = tr("Latest activity: ") + WPNetworkEngine::LocalizedTime(latestActivity);;
}

WPBlogGraphicsListItem::~WPBlogGraphicsListItem()
{

}

void WPBlogGraphicsListItem::paintContent(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	painter->setPen(Qt::black);

	painter->setFont(*iThemeData.iListTitleFont);
	painter->drawText(iGeometry.iTitle, iTitle);

	painter->setFont(*iThemeData.iListInfoFont);
	painter->drawText(iGeometry.iStatus, iStatus);
	painter->drawText(iGeometry.iActivity, iActivity);
}

bool WPBlogGraphicsListItem::IsBusy()
{
	return false;
}

bool WPBlogGraphicsListItem::HasError()
{
	return false;
}

void WPBlogGraphicsListItem::handleGeomeryChange()
{
    iGeometry.update(size(), iThemeData);
    
    calculateElidedTexts();

    setMinimumHeight(iGeometry.height());
}

const WPDataBlog& WPBlogGraphicsListItem::getBlog() const
{
	return iBlog;
}

void WPBlogGraphicsListItem::calculateElidedTexts()
{
	QFontMetrics titleFm(*iThemeData.iListTitleFont);
	iTitle = titleFm.elidedText(iBlog.iBlogName, Qt::ElideRight, iGeometry.iTitle.width());
}

void WPBlogGraphicsListItem::Geometry::update (const QSizeF &size, const ThemeData &theme)
{
    if (iCachedSize == size)
        // fresh
        return;

    else
        iCachedSize = size;

    // XXX: recalc boundingRect by hand
    int itemPadding = theme.iListItemPadding;
    int padding = theme.iListItemShadowMargin + itemPadding;
    int width = size.width() - 2 * padding;
    
    // Font metrics
    QFontMetrics titleFont (*theme.iListTitleFont);
    QFontMetrics infoFont (*theme.iListInfoFont);
    QFontMetrics detailFont (*theme.iListDetailFont);

    iTitle.setRect(padding, padding, width, titleFont.height());
    iStatus.setRect(padding, iTitle.bottom() + itemPadding, width, infoFont.height());
    iActivity.setRect(padding, iStatus.bottom() + 2 * itemPadding, width, detailFont.height());

    iHeight = iActivity.bottom() + 2 * itemPadding + padding;
}

