#include "wppostgraphicslistitem.h"
#include "WPData.h"
#include "themedatastorage.h"
#include "HtmlPainter.h"

#include <QPoint>
#include <QRect>
#include <QPainter>

// static member
WPPostGraphicsListItem::Geometry WPPostGraphicsListItem::iGeometry;

WPPostGraphicsListItem::WPPostGraphicsListItem(WPPost aPost, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
    : WPGraphicsListItem(aBusyIndicator, aThemeData)
    , iPost(aPost)
{
    setObjectName(QString::fromLatin1("WPPostListItem"));
    iDate = aPost->GetShortLocalTimeString();

    int commentCount = iPost->GetComments().count();

    if (commentCount == 1) {
        iCommentsText = tr("1 comment");
    } else {
        iCommentsText = tr("%1 comments") .arg(commentCount);
    }
    
    iTitle = new HtmlPainter(HtmlPainter::Title, true);
    iTitle->setContent(iPost->iTitle);
}

WPPostGraphicsListItem::~WPPostGraphicsListItem()
{
}

bool WPPostGraphicsListItem::operator<(WPPostGraphicsListItem const & other)
{
    return (iPost->GetLocalTime() <other.iPost->GetLocalTime()); 
}

WPPost WPPostGraphicsListItem::GetPost() const
{
    return iPost;
}

void WPPostGraphicsListItem::paintContent(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget* /*widget*/)
{
    painter->setPen(Qt::black);

    iTitle->setWidthAndMode(iGeometry.iTitle.width(), true);
    iTitle->paint(painter, iGeometry.iTitle);

    painter->setFont(*iThemeData.iListInfoFont);
    painter->drawText(iGeometry.iAuthor, "by " + iPost->iWpAuthorDisplayName);

    painter->setFont(*iThemeData.iListDetailFont);
    painter->drawText(iGeometry.iDate, iDate);
    painter->drawText(iGeometry.iCommentInfo, iCommentsText);
}

bool WPPostGraphicsListItem::IsBusy()
{
    return iPost->IsBusy();
}

bool WPPostGraphicsListItem::HasError()
{
    return iPost->iError;
}

void WPPostGraphicsListItem::handleGeomeryChange()
{
    iGeometry.update(*this);
    setMinimumHeight(iGeometry.height());
}
        
void WPPostGraphicsListItem::Geometry::update (WPPostGraphicsListItem & parent)
{
    const QSizeF size = parent.size();
    const ThemeData &theme = parent.iThemeData;
    
    if (iCachedSize == size)
        // fresh
        return;

    else
        iCachedSize = size;

    // XXX: recalc boundingRect by hand
    int itemPadding = theme.iListItemPadding;
    int padding = theme.iListItemShadowMargin + itemPadding;
    int width = size.width() - 2 * padding;
    
    // Font metrics
    QFontMetrics infoFont (*theme.iListInfoFont);
    QFontMetrics detailFont (*theme.iListDetailFont);

    iTitle.setRect(padding, padding, width, parent.iTitle->fontMetrics().height());
    iAuthor.setRect(padding, iTitle.bottom() + itemPadding, width, infoFont.height());
    iDate.setRect(padding, iAuthor.bottom() + 3 * itemPadding, width, detailFont.height());
    iCommentInfo.setRect(padding, iDate.bottom() + itemPadding, width, detailFont.height());

    iHeight = iCommentInfo.bottom() + padding;
}
